from .files import FileClient


def show(gcs_path: str) -> list[str]:
    """Backported dapla function to recursively show all folders below a given GCS path.

    Args:
        gcs_path: the path from which you want to list all folders

    Returns:
        A simplified list of files or folders
    """
    fs = FileClient.get_gcs_file_system()
    out = dict()
    files = None
    for _path, dirs, files in fs.walk(gcs_path, detail=True):  # noqa: B007
        out.update({_trimmed_name(info): info for name, info in dirs.items()})
    # Add the base path (if it exists) to avoid an empty list when there are no subfolders
    if len(out) == 0 and files is not None:
        # Get the bucket name from any of the files
        bucket_name = next(iter(files.values()))["bucket"]
        trimmed_name = gcs_path.replace("gs://", "").lstrip(bucket_name).rstrip("/")
        out[trimmed_name] = {}
    return sorted(out)


def details(gcs_path: str) -> list[dict[str, str]]:
    """Backported dapla function to support detailed list of files for a given GCS path.

    Args:
        gcs_path: the path from which you want to list all folders

    Returns:
        A list of dicts containing file details
    """
    fs = FileClient.get_gcs_file_system()
    return list(
        map(
            lambda o: (
                _folder_item(o) if o["storageClass"] == "DIRECTORY" else _file_item(o)
            ),
            fs.ls(gcs_path, detail=True),
        )
    )


def _trimmed_name(o: dict[str, str]) -> str:
    return o["name"].lstrip(o["bucket"])


def _folder_item(o: dict[str, str]) -> dict[str, str]:
    return {"Size": o["size"], "Name": _trimmed_name(o)}


def _file_item(o: dict[str, str]) -> dict[str, str]:
    return {
        "Size": o["size"],
        "Name": _trimmed_name(o),
        "Created": o["timeCreated"],
        "Updated": o["updated"],
    }



import json
import logging
import os
import typing as t
from collections.abc import Sequence
from datetime import datetime
from datetime import timedelta
from functools import lru_cache
from functools import partial
from typing import Any
from typing import Optional

import google.auth
import requests
from google.auth.transport.requests import Request as GoogleAuthRequest
from google.oauth2.credentials import Credentials
from IPython.display import HTML
from IPython.display import display
from jupyterhub.services.auth import HubAuth

from dapla.const import DaplaEnvironment
from dapla.const import DaplaRegion
from dapla.const import DaplaService

logger = logging.getLogger(__name__)

# Refresh window was modified in: https://github.com/googleapis/google-auth-library-python/commit/c6af1d692b43833baca978948376739547cf685a
# The change was directed towards high latency environments, and should not apply to us.
# Since we can't force a refresh, the threshold is lowered to keep us from waiting ~4 minutes for a new token.
# A permanent fix would be to supply credentials with a refresh endpoint
# that allways returns a token that is valid for more than 3m 45s.
google.auth._helpers.REFRESH_THRESHOLD = timedelta(seconds=20)


class AuthClient:
    """Client for retrieving authentication information."""

    @staticmethod
    def _get_current_dapla_metadata() -> (
        tuple[Optional[DaplaEnvironment], Optional[DaplaService], Optional[DaplaRegion]]
    ):
        try:
            env = DaplaEnvironment(os.getenv("DAPLA_ENVIRONMENT"))
        except ValueError:
            env = None

        try:
            service = DaplaService(os.getenv("DAPLA_SERVICE"))
        except ValueError:
            service = None

        try:
            region = DaplaRegion(os.getenv("DAPLA_REGION"))
        except ValueError:
            region = None

        return env, service, region

    @staticmethod
    def get_dapla_region() -> Optional[DaplaRegion]:
        """Checks if the current Dapla Region is Dapla Lab."""
        env, service, region = AuthClient._get_current_dapla_metadata()
        return region

    @staticmethod
    def _refresh_handler(
        request: google.auth.transport.Request,
        scopes: Sequence[str],
        from_jupyterhub: bool = False,
    ) -> tuple[str, datetime]:
        # We manually override the refresh_handler method with our custom logic for fetching tokens.
        # Previously, we directly overrode the `refresh` method. However, this
        # approach led to deadlock issues in gcsfs/credentials.py's maybe_refresh method.
        return AuthClient.fetch_google_token(from_jupyterhub=from_jupyterhub)

    @staticmethod
    def fetch_google_token_from_oidc_exchange(
        request: GoogleAuthRequest, _scopes: Sequence[str]
    ) -> tuple[str, datetime]:
        """Fetches the Google token by exchanging an OIDC token.

        Args:
            request: The GoogleAuthRequest object.
            _scopes: The scopes to request.

        Raises:
            AuthError: If the request to the OIDC token exchange endpoint fails.

        Returns:
            A tuple of (google-token, expiry).
        """
        if os.getenv("OIDC_TOKEN_EXCHANGE_URL") is None:
            raise AuthError(
                "env variable  'OIDC_TOKEN_EXCHANGE_URL' was not found when"
                "attempting token exchange with OIDC endpoint"
            )
        response = request.__call__(
            url=os.environ["OIDC_TOKEN_EXCHANGE_URL"],
            method="POST",
            body={
                "subject_token": os.environ["OIDC_TOKEN"],
                "grant_type": "urn:ietf:params:oauth:grant-type:token-exchange",
                "requested_token_type": "urn:ietf:params:oauth:token-type:access_token",
                "requested_issuer": "google",
                "client_id": "onyxia-api",
            },
        )
        if response.status == 200:
            auth_data = json.loads(response.data)
            expiry = datetime.utcnow() + timedelta(seconds=auth_data["expires_in"])
            return auth_data["access_token"], expiry
        else:
            error = json.loads(response.data)
            print("Error: ", error["error_description"])
            raise AuthError

    @staticmethod
    def fetch_google_token_from_jupyter() -> str:
        """Fetches the personal access token for the current user.

        Raises:
            AuthError: If the token exchange request to JupyterHub fails.

        Returns:
            The personal access token.
        """
        try:
            google_token = AuthClient.fetch_local_user_from_jupyter()[
                "exchanged_tokens"
            ]["google"]["access_token"]
            return t.cast(str, google_token)
        except AuthError as err:
            err._print_warning()
            raise err

    @staticmethod
    def fetch_local_user_from_jupyter() -> dict[str, Any]:
        """Retrieves user information, most notably access tokens for use in authentication.

        Raises:
            AuthError: If the request to the user endpoint fails.

        Returns:
            The user data from the token .
        """
        if os.getenv("LOCAL_USER_PATH") is None:
            raise AuthError(
                "env variable LOCAL_USER_PATH was not found when"
                "attempting token exchange with JupyterLab"
            )

        # Helps getting the correct ssl configs
        hub = HubAuth()
        response = requests.get(
            os.environ["LOCAL_USER_PATH"],
            headers={"Authorization": f"token {hub.api_token}"},
            cert=(hub.certfile, hub.keyfile),
            verify=hub.client_ca,
            allow_redirects=False,
        )
        if response.status_code == 200:
            user_data = response.json()
            return t.cast(dict[str, Any], user_data)
        else:
            raise AuthError

    @staticmethod
    def fetch_google_token(
        request: Optional[GoogleAuthRequest] = None,
        scopes: Optional[Sequence[str]] = None,
        from_jupyterhub: bool = False,
    ) -> tuple[str, datetime]:
        """Fetches the Google token for the current user.

        Scopes in the argument is ignored, but are kept for compatibility
        with the Credentials refresh handler method signature.

        Args:
            request: The GoogleAuthRequest object.
            scopes: The scopes to request.
            from_jupyterhub: Whether the google token should be exchanged from JupyterHub.
                if false, exchange from an OIDC endpoint decided by OIDC_TOKEN_EXCHANGE_URL.

        Raises:
            AuthError: If the token exchange.

        Returns:
            The Google token.
        """
        try:
            if from_jupyterhub is True:
                user_info = AuthClient.fetch_local_user_from_jupyter()
                google_token = user_info["exchanged_tokens"]["google"]["access_token"]
                expiry = datetime.utcfromtimestamp(
                    user_info["exchanged_tokens"]["google"]["exp"]
                )

            else:
                if request is None:
                    request = GoogleAuthRequest()
                if scopes is None:
                    scopes = []

                google_token, expiry = AuthClient.fetch_google_token_from_oidc_exchange(
                    request, scopes
                )

        except AuthError as err:
            err._print_warning()
            raise err

        return google_token, expiry

    @staticmethod
    def fetch_google_credentials(force_token_exchange: bool = False) -> Credentials:
        """Fetches the Google credentials for the current user.

        Args:
            force_token_exchange: Forces authentication by token exchange.

        Raises:
            AuthError: If fails to fetch credentials.

        Returns:
            The Google "Credentials" object.
        """
        env, service, region = AuthClient._get_current_dapla_metadata()
        force_token_exchange = (
            os.getenv("DAPLA_TOOLBELT_FORCE_TOKEN_EXCHANGE") == "1"
            or force_token_exchange
        )
        try:
            match (env, service, region):
                case (_, _, _) if force_token_exchange is True:
                    logger.debug("Auth - Forced token exchange")
                    token, expiry = AuthClient.fetch_google_token()
                    credentials = Credentials(
                        token=token,
                        expiry=expiry,
                        token_uri="https://oauth2.googleapis.com/token",
                        refresh_handler=AuthClient._refresh_handler,
                    )
                case (_, DaplaService.CLOUD_RUN, _):
                    logger.debug("Auth - Cloud Run detected, using ADC")
                    credentials, _ = google.auth.default()
                case (
                    _,
                    DaplaService.JUPYTERLAB,
                    DaplaRegion.ON_PREM | DaplaRegion.BIP,
                ):
                    logger.debug("Auth - JupyterLab detected, using token exchange")
                    token, expiry = AuthClient.fetch_google_token(from_jupyterhub=True)
                    credentials = Credentials(
                        token=token,
                        expiry=expiry,
                        token_uri="https://oauth2.googleapis.com/token",
                        refresh_handler=partial(
                            AuthClient._refresh_handler, from_jupyterhub=True
                        ),
                    )
                case (_, _, DaplaRegion.DAPLA_LAB):
                    logger.debug("Auth - Dapla Lab detected, attempting to use ADC")
                    adc_env = os.getenv("DAPLA_GROUP_CONTEXT")
                    if adc_env is None:
                        raise AuthError(
                            "Dapla Group selection feature is not enabled. "
                            "This is necessary in order to access buckets in Dapla Lab. "
                            "The feature needs to be enabled *before* starting the service, "
                            "and can be done in the 'Buckets' configuration tab"
                        )
                    logger.debug(
                        "Auth - 'DAPLA_GROUP_CONTEXT' env variable is set, "
                        f"using ADC as group {adc_env}"
                    )
                    credentials, _ = google.auth.default()
                case (_, _, _):
                    logger.debug("Auth - Default authentication used (ADC)")
                    credentials, _ = google.auth.default()
        except AuthError as err:
            err._print_warning()
            raise err

        return credentials

    @staticmethod
    def fetch_personal_token() -> str:
        """If Dapla Region is Dapla Lab, retrieve the OIDC token/Keycloak token from the environment.

        Returns:
            str: The OIDC token.

        Raises:
            MissingConfigurationException: If the OIDC_TOKEN environment variable is missing or is not set.

        If Dapla Region is BIP, retrieve the Keycloak token jupyterhub.

        Returns:
            str: personal/keycloak token.

        Raises:
            AuthError: Handles AuthError.
        """
        env, service, region = AuthClient._get_current_dapla_metadata()
        if region == DaplaRegion.DAPLA_LAB:
            logger.debug("Auth - Dapla Lab detected, using OIDC_TOKEN")
            keycloak_token = os.getenv("OIDC_TOKEN")
            if not keycloak_token:
                raise MissingConfigurationException("OIDC_TOKEN")
            else:
                return keycloak_token
        else:
            logger.debug("Auth - BIP detected, using jupyterhub personal token")
            try:
                personal_token = AuthClient.fetch_local_user_from_jupyter()[
                    "access_token"
                ]
                return t.cast(str, personal_token)
            except AuthError as err:
                err._print_warning()
                raise err

    @staticmethod
    @lru_cache(maxsize=1)
    def fetch_email_from_credentials() -> Optional[str]:
        """Retrieves an e-mail based on current Google Credentials. Potentially makes a Google API call."""
        if os.getenv("DAPLA_REGION") == str(DaplaRegion.DAPLA_LAB):
            return os.getenv("DAPLA_USER")

        credentials = AuthClient.fetch_google_credentials()
        response = requests.get(
            url=f"https://oauth2.googleapis.com/tokeninfo?access_token={credentials.token}"
        )

        return response.json().get("email") if response.status_code == 200 else None


class AuthError(Exception):
    """This exception class is used when the communication with the custom auth handler fails.

    This is normally due to stale auth session.
    """

    def _print_warning(self) -> None:
        (
            display(
                HTML(
                    'Your session has timed out. Please <a href="/hub/login">log in</a> to continue.'
                )
            )
        )


class MissingConfigurationException(Exception):
    """Exception raised when a required environment variable or configuration is missing."""

    def __init__(self, variable_name: str) -> None:
        """Initializes a new instance of the MissingConfigurationException class.

        Args:
        variable_name (str): The name of the missing environment variable or configuration.
        message (str): The error message to be displayed. Defaults to an empty string.
        """
        self.variable_name = variable_name
        self.message = f"Missing required environment variable: {variable_name}"
        super().__init__(self.message)

    def __str__(self) -> str:
        """Returns a string representation of the exception."""
        return f"Configuration error: {self.message}"



from __future__ import annotations

import typing as t
from enum import Enum
from typing import Any
from typing import Optional

# import this module to trigger import side-effect and register the pyarrow extension types
import pandas.core.arrays.arrow.extension_types  # type: ignore [import-untyped] # noqa: F401
import pyarrow.compute
from google.oauth2.credentials import Credentials
from pandas import DataFrame
from pandas import Series
from pandas import read_csv
from pandas import read_excel
from pandas import read_fwf
from pandas import read_json
from pandas import read_sas
from pandas import read_xml

from .auth import AuthClient
from .files import FileClient


class SupportedFileFormat(Enum):
    """A collection of supported file formats."""

    PARQUET = "parquet"
    JSON = "json"
    CSV = "csv"
    FWF = "fwf"
    XML = "xml"
    SAS7BDAT = "sas7bdat"
    EXCEL = "excel"

    @classmethod
    def _missing_(cls, value: object) -> None:
        raise ValueError(
            f"{value} is not a valid file format. Valid formats: %s"
            % (", ".join([repr(m.value) for m in cls]))
        )


def read_pandas(
    gcs_path: str | list[str],
    file_format: Optional[str] = "parquet",
    columns: Optional[list[str]] = None,
    filters: Optional[
        list[tuple[Any] | list[tuple[Any]]] | pyarrow.compute.Expression
    ] = None,
    **kwargs: Any,
) -> t.Union[DataFrame, Series]:  # type: ignore [type-arg]
    """Convenience method for reading a dataset from a given GCS path and convert it to a Pandas dataframe.

    Args:
        gcs_path: Path or paths to the directory or file you want to get the contents of.
            Support multiple paths if reading parquet format.
        file_format: The expected file format. All file formats other than "parquet" are delegated to Pandas
            methods like read_json, read_csv, etc. Defaults to "parquet".
        columns: Choose specific columsn to read. Defaults to None.
        filters: Add row filter to process when reading parquet. The filter
            should follow pyarrow methods. See examples in the docs:
            https://arrow.apache.org/docs/python/generated/pyarrow.parquet.ParquetDataset.html#pyarrow.parquet.ParquetDataset.
            Defaults to None.
        kwargs: Additional arguments to pass to the underlying Pandas "read_*()" method.

    Raises:
        ValueError: If multiple paths are provided for non-parquet formats.

    Returns:
        A Pandas DataFrame containing the selected dataset.

    """
    if isinstance(gcs_path, str):
        gcs_path = FileClient._ensure_gcs_uri_prefix(gcs_path)

    if isinstance(gcs_path, list) and file_format != "parquet":
        raise ValueError("Multiple paths are only supported for parquet format")
    match SupportedFileFormat(file_format):
        case SupportedFileFormat.PARQUET:
            import pyarrow.parquet as pq

            fs = FileClient.get_gcs_file_system()

            # Workaround for https://github.com/apache/arrow/issues/30481
            if isinstance(gcs_path, list):
                gcs_path = [
                    FileClient._remove_gcs_uri_prefix(file) for file in gcs_path
                ]
            else:
                gcs_path = FileClient._remove_gcs_uri_prefix(gcs_path)

            parquet_ds = pq.ParquetDataset(
                gcs_path,  # type: ignore [arg-type]
                filesystem=fs,
                filters=filters,  # type: ignore [arg-type]
            )  # Stubs show the incorrect type -
            # see https://arrow.apache.org/docs/python/generated/pyarrow.parquet.ParquetDataset.html

            return parquet_ds.read_pandas(columns=columns).to_pandas(
                split_blocks=False, self_destruct=True, **kwargs
            )
        case SupportedFileFormat.JSON:
            assert isinstance(gcs_path, str)
            return t.cast(
                "DataFrame | Series[Any]",
                read_json(gcs_path, storage_options=_get_storage_options(), **kwargs),
            )
        case SupportedFileFormat.CSV:
            assert isinstance(gcs_path, str)
            return t.cast(
                "DataFrame | Series[Any]",
                read_csv(gcs_path, storage_options=_get_storage_options(), **kwargs),
            )
        case SupportedFileFormat.FWF:
            assert isinstance(gcs_path, str)
            return t.cast(
                "DataFrame | Series[Any]",
                read_fwf(gcs_path, storage_options=_get_storage_options(), **kwargs),
            )
        case SupportedFileFormat.XML:
            assert isinstance(gcs_path, str)
            return t.cast(
                "DataFrame | Series[Any]",
                read_xml(gcs_path, storage_options=_get_storage_options(), **kwargs),
            )
        case SupportedFileFormat.EXCEL:
            assert isinstance(gcs_path, str)

            return t.cast(
                "DataFrame | Series[Any]",
                read_excel(gcs_path, storage_options=_get_storage_options(), **kwargs),
            )
        case SupportedFileFormat.SAS7BDAT:
            assert isinstance(gcs_path, str)

            fs = FileClient.get_gcs_file_system()

            with fs.open(gcs_path) as sas:
                df = read_sas(sas, format="sas7bdat", **kwargs)

            return t.cast("DataFrame | Series[Any]", df)
        case _:
            raise ValueError(f"Invalid file format {file_format}")


def write_pandas(
    df: DataFrame, gcs_path: str, file_format: str = "parquet", **kwargs: Any
) -> None:
    """Convenience method for writing a Pandas DataFrame to a given GCS path.

    Args:
        df: The Pandas DataFrame to write to file.
        gcs_path: The GCS path to the destination file. Must have an extension that corresponds to the file_format
        file_format: The expected file format. All file formats other than "parquet" are delegated to Pandas
        **kwargs: Additional arguments to pass to the underlying Pandas "to_*()" method.

    Raises:
        ValueError: If the file format is invalid.
        ValueError: If the path does not have an extension that corresponds to the file format.
    """
    import pyarrow.parquet

    if isinstance(gcs_path, str):
        gcs_path = FileClient._ensure_gcs_uri_prefix(gcs_path)

    match SupportedFileFormat(file_format):
        case SupportedFileFormat.PARQUET:
            # Transfom and write pandas dataframe
            from_pandas_kwargs = {"schema": kwargs.pop("schema", None)}
            table = pyarrow.Table.from_pandas(
                df, preserve_index=True, **from_pandas_kwargs
            )
            if ".parquet" not in gcs_path:
                raise ValueError("Path must be a parquet file")
            fs = FileClient.get_gcs_file_system()
            with fs.open(gcs_path, mode="wb") as buffer:
                pyarrow.parquet.write_table(
                    table,
                    buffer,
                    flavor="spark",
                    compression="snappy",
                    coerce_timestamps="ms",
                    **kwargs,
                )
        case SupportedFileFormat.JSON:
            df.to_json(gcs_path, **kwargs, storage_options=_get_storage_options())  # type: ignore [call-overload]
        case SupportedFileFormat.CSV:
            df.to_csv(gcs_path, storage_options=_get_storage_options(), **kwargs)
        case SupportedFileFormat.XML:
            df.to_xml(gcs_path, storage_options=_get_storage_options(), **kwargs)
        case SupportedFileFormat.EXCEL:
            # FIXME: mypy complains about `storage_options` being an unknown argument
            df.to_excel(gcs_path, storage_options=_get_storage_options(), **kwargs)  # type: ignore [call-arg]
        case SupportedFileFormat.FWF:
            raise ValueError("Writing with fixed width format is not supported")
        case SupportedFileFormat.SAS7BDAT:
            raise ValueError(
                "Writing to SAS7BDAT is not supported since it's a proprietary format"
            )
        case _:
            raise ValueError(f"Invalid file format {file_format}")


def _get_storage_options() -> Optional[dict[str, Optional[Credentials]]]:
    """Returns the ``storage_options`` that are used in Pandas for specifying extra options for a particular storage connection that will be parsed by ``fsspec``.

    In this case when the URL starts with "gcs://".
    An error will be raised by Pandas if providing this argument with a local path or a file-like buffer.
    See the fsspec and backend storage implementation docs for the set of allowed keys and values
    """
    credentials = AuthClient.fetch_google_credentials()
    return {"token": credentials} if credentials is not None else None



import os

import requests
from jupyterhub.services.auth import HubAuth


def generate_api_token(
    expires_in: int = 3600, description: str = "Generated API token from Dapla Toolbelt"
) -> dict[str, str]:
    """Generate a new API token for the logged in Jupyterhub user.

    Such tokens can be used by third party applications to connect to Jupyterhub running remotely.
    Examples are IDEs like VSCode or Pycharm.
    :param expires_in: number of seconds until the token expires
    :param description: optional description of the token
    :return: a dict that contains the token value, and token URL
    """
    hub = HubAuth()
    body = {"expires_in": expires_in, "note": description}
    hub_response = requests.post(
        os.environ["JUPYTERHUB_API_URL"]
        + "/users/"
        + os.environ["JUPYTERHUB_USER"]
        + "/tokens",
        json=body,
        headers={"Authorization": f"token {hub.api_token}"},
        allow_redirects=False,
    )
    hub_response.raise_for_status()
    return {
        "token": hub_response.json()["token"],
        "token_url": os.environ["JUPYTERHUB_SERVICE_PREFIX"]
        + "?token="
        + hub_response.json()["token"],
    }



"""Dapla Toolbelt collection."""

import importlib.metadata
import warnings

import tomli

from .auth import AuthClient
from .backports import details
from .backports import show
from .collector import CollectorClient
from .converter import ConverterClient
from .doctor import Doctor
from .files import FileClient
from .git import repo_root_dir
from .gsm import get_secret_version
from .guardian import GuardianClient
from .jupyterhub import generate_api_token
from .pandas import read_pandas
from .pandas import write_pandas
from .pubsub import trigger_source_data_processing

__all__ = [
    "AuthClient",
    "details",
    "show",
    "CollectorClient",
    "ConverterClient",
    "Doctor",
    "FileClient",
    "repo_root_dir",
    "GuardianClient",
    "generate_api_token",
    "read_pandas",
    "write_pandas",
    "trigger_source_data_processing",
    "get_secret_version",
]


# Register function for second try: getting from pyproject directly (when running under pytest for example)
def _try_getting_pyproject_toml(e: Exception | None = None) -> str:
    """Look for version in pyproject.toml, if not found, set to 0.0.0 ."""
    if e is None:
        passed_excep: Exception = Exception("")
    else:
        passed_excep = e
    try:
        with open("../pyproject.toml", "rb") as f:
            toml_dict = tomli.load(f)
        version: str = toml_dict["tool"]["poetry"]["version"]
        return version
    except Exception as e:
        version_missing: str = "0.0.0"
        warn_msg = f"Error from dapla-toolbelt __init__, not able to get version-number, setting it to {version_missing}: {e}, passed in from importlib: {passed_excep}"
        warnings.warn(warn_msg, category=Warning, stacklevel=2)
        return version_missing


# First try is setting __version__ attribute programatically from installed package
try:
    __version__ = importlib.metadata.version("dapla-toolbelt")
except importlib.metadata.PackageNotFoundError as e:
    __version__ = _try_getting_pyproject_toml(e)



from typing import Optional

from google.cloud.secretmanager import SecretManagerServiceClient

from .auth import AuthClient


def get_secret_version(
    project_id: str, shortname: str, version_id: Optional[str] = "latest"
) -> str:
    """Access the payload for a given secret version.

    The user's google credentials are used to authorize that the user have permission
    to access the secret_id.

    Args:
        project_id (str): ID of the Google Cloud project where the secret is stored.
        shortname (str): Name (not full path) of the secret in Secret Manager.
        version_id (str, optional): The version of the secret to access. Defaults to 'latest'.

    Returns:
        str: The payload of the secret version as a UTF-8 decoded string.
    """
    client = SecretManagerServiceClient(
        credentials=AuthClient.fetch_google_credentials()
    )
    secret_name = f"projects/{project_id}/secrets/{shortname}/versions/{version_id}"
    response = client.access_secret_version(name=secret_name)
    return str(response.payload.data.decode("UTF-8"))



from enum import Enum


class DaplaEnvironment(Enum):
    """Represents the 'DAPLA_ENVIRONMENT' environment variable."""

    DEV = "DEV"
    STAGING = "STAGING"
    TEST = "TEST"
    PROD = "PROD"


class DaplaService(Enum):
    """Represents the 'DAPLA_SERVICE' environment variable."""

    JUPYTERLAB = "JUPYTERLAB"
    VS_CODE = "VS_CODE"
    R_STUDIO = "R_STUDIO"
    CLOUD_RUN = "CLOUD_RUN"


class DaplaRegion(Enum):
    """Represents the 'DAPLA_REGION' environment variable."""

    ON_PREM = "ON_PREM"
    DAPLA_LAB = "DAPLA_LAB"
    BIP = "BIP"
    CLOUD_RUN = "CLOUD_RUN"
